<?php

/**
 * @file
 * Implements hooks to update the module.
 *
 * Note that comments on update functions are parsed and shown to the
 * site admin to describe an update before it is applied.
 */

use Drupal\Core\Database\Database;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\user\Entity\Role;

use Drupal\foldershare\Constants;
use Drupal\foldershare\Settings;
use Drupal\foldershare\Utilities\ConfigurationUtilities;
use Drupal\foldershare\Utilities\LimitUtilities;
use Drupal\foldershare\Entity\FolderShare;
use Drupal\foldershare\Entity\FolderShareScheduledTask;
use Drupal\foldershare\ManageUsageStatistics;
use Drupal\foldershare\ManageSearch;

/*----------------------------------------------------------------------
 *
 * 0.5.0 to 0.5.1
 *
 *----------------------------------------------------------------------*/

/**
 * Update the configurations for views, search, and entity views.
 *
 * These updates support a substantial overhaul of the user
 * interface that revises the module's views, entity view page, and search
 * indexing. Upon completion, the search index will be cleared and all site
 * caches flushed.
 */
function foldershare_update_8001(&$sandbox) {
  // Restore the search configuration and clear the index.
  if (\Drupal::service('module_handler')->moduleExists('search') === TRUE) {
    ConfigurationUtilities::revertConfiguration(
      'core',
      'search.page.foldershare_search');

    // DEPRECATED. Remove this when Drupal 8.7 is no longer supported.
    $versionParts = explode('.', \Drupal::VERSION);
    $drupalVersion = floatval($versionParts[0] . '.' . $versionParts[1]);
    if ($drupalVersion < 8.8) {
      // Pre-Drupal 8.8, the 'search.index' service does not exist.
      // Call the deprecated search_index_clear() function.
      call_user_func('search_index_clear', Constants::SEARCH_INDEX);
    }
    // END DEPRECATED.
    else {
      \Drupal::service('search.index')->clear(Constants::SEARCH_INDEX);
    }
  }

  // Restore the view configuration.
  ConfigurationUtilities::revertConfiguration(
    'view',
    Constants::VIEW_LISTS);

  // Restore the entity display configuration.
  ConfigurationUtilities::revertConfiguration(
    'core',
    'entity_view_display.foldershare.foldershare.default');
}

/*----------------------------------------------------------------------
 *
 * 0.5.4 to 0.6.0
 *
 *----------------------------------------------------------------------*/

/**
 * Update entity structure, entity view, and views tables.
 *
 * This update removes obsolete internal entity fields, adds new
 * internal entity fields, removes a text field formatter, adds new internal
 * field formatters, adds new menu commands, and adds a share-with-public
 * permission.
 *
 * Note that the entity view page and views tables are necessarily reset.
 * Any site customizations will be lost.
 */
function foldershare_update_8002() {
  $updateManager = \Drupal::entityDefinitionUpdateManager();
  $connection    = Database::getConnection();
  $schema        = $connection->schema();
  $baseTable     = FolderShare::BASE_TABLE;
  $usageTable    = ManageUsageStatistics::USAGE_TABLE;

  //
  // Update all entities with 'rootfolder' to use 'folder' instead.
  // --------------------------------------------------------------
  // For all entities that use the old value, update to the new value.
  $connection->update($baseTable)
    ->condition('kind', 'rootfolder', '=')
    ->fields(['kind' => FolderShare::FOLDER_KIND])
    ->execute();

  //
  // Remove 'nRootFolders' column from usage table.
  // ----------------------------------------------
  // Remove the column.
  if ($schema->fieldExists($usageTable, 'nRootFolders') === TRUE) {
    $schema->dropField($usageTable, 'nRootFolders');
  }

  //
  // Add 'systemhidden' and 'systemdisabled' fields.
  // -----------------------------------------------
  // Update the base field definition. These definitions are copied from
  // the entity's base field definition for new installs.
  if ($schema->fieldExists($baseTable, 'systemhidden') === FALSE) {
    // System hidden field flag.
    // - Boolean.
    // - Default is FALSE.
    // - 'view' unsupported.
    // - 'form' unsupported.
    //
    // Field editing blocked by FolderShareAccessControlHandler for all users.
    $definition = BaseFieldDefinition::create('boolean')
      ->setLabel(t('System hidden'))
      ->setDescription(t('A TRUE/FALSE value indicating if the entity has been hidden by the system during special operations.'))
      ->setRequired(FALSE)
      ->setDefaultValue(FALSE)
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE);

    // Add the field.
    $updateManager->installFieldStorageDefinition(
      'systemhidden',
      FolderShare::ENTITY_TYPE_ID,
      FolderShare::ENTITY_TYPE_ID,
      $definition);
  }

  if ($schema->fieldExists($baseTable, 'systemdisabled') === FALSE) {
    // System disabled field flag.
    // - Boolean.
    // - Default is FALSE.
    // - 'view' unsupported.
    // - 'form' unsupported.
    //
    // Field editing blocked by FolderShareAccessControlHandler for all users.
    $definition = BaseFieldDefinition::create('boolean')
      ->setLabel(t('System disabled'))
      ->setDescription(t('A TRUE/FALSE value indicating if the entity has been disabled by the system during special operations.'))
      ->setRequired(FALSE)
      ->setDefaultValue(FALSE)
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', FALSE);

    // Add the field.
    $updateManager->installFieldStorageDefinition(
      'systemdisabled',
      FolderShare::ENTITY_TYPE_ID,
      FolderShare::ENTITY_TYPE_ID,
      $definition);
  }

  //
  // Update all entities to default 'systemhidden' and 'systemdisabled' values.
  // --------------------------------------------------------------------------
  // Default to FALSE.
  $connection->update(FolderShare::BASE_TABLE)
    ->fields([
      'systemhidden'   => (int) 0,
      'systemdisabled' => (int) 0,
    ])
    ->execute();

  //
  // Remove 'grantdisableduids' field.
  // ---------------------------------
  // Update the base field definition.
  $definition = $updateManager->getFieldStorageDefinition(
    'grantdisableduids',
    FolderShare::ENTITY_TYPE_ID);
  if ($definition !== NULL) {
    $updateManager->uninstallFieldStorageDefinition($definition);

    // Uninstallling a field marks the field as deleted data. Future
    // runs of CRON will delete the field data in batches until all
    // entities have been updated, and then the multi-value table
    // will be automatically dropped.
  }

  //
  // Flush all caches.
  // -----------------
  // Render and views now use 'systemhidden' and 'systemdisabled', so their
  // caches need to be flushed. The breadcrumb builder's interface has changed,
  // and the field formatter plugins have been updated. There are also new
  // menu commands and a new permission.
  //
  // The cache must be flushed now before we install configurations that use
  // the new field formatters, use the new menu commands, or use the new
  // permission.
  drupal_flush_all_caches();

  //
  // Reset views.
  // ------------
  // The new views use new field formatters, check the 'systemhidden' flag,
  // and support lists in dialogs.
  ConfigurationUtilities::revertConfiguration(
    'view',
    Constants::VIEW_LISTS);

  //
  // Reset entity view.
  // ------------------
  // The previous release included a text display field formatter that has
  // been removed.
  ConfigurationUtilities::revertConfiguration(
    'core',
    'entity_view_display.foldershare.foldershare.default');

  //
  // Enable new menu commands.
  // -------------------------
  // Insure the new menu commands are enabled by default.
  $currentCommands = Settings::getCommandMenuAllowed();
  if (in_array('foldersharecommand_delete_as_admin', $currentCommands) === FALSE) {
    $currentCommands[] = 'foldersharecommand_delete_as_admin';
  }

  if (in_array('foldersharecommand_delete_on_rootlist', $currentCommands) === FALSE) {
    $currentCommands[] = 'foldersharecommand_delete_on_rootlist';
  }

  if (in_array('foldersharecommand_release_share', $currentCommands) === FALSE) {
    $currentCommands[] = 'foldersharecommand_release_share';
  }

  Settings::setCommandMenuAllowed($currentCommands);

  //
  // Enable the share-with-public permission for roles with share permission.
  // ------------------------------------------------------------------------
  // The old "share" permission shared with others or public. The new one only
  // shares with others, and a new "share with public" permission shares with
  // the public. If user roles had the old "share" permission, give them both
  // share permissions now.
  foreach (\Drupal::entityQuery('user_role')->execute() as $roleId) {
    $role = Role::load($roleId);
    if ($role !== NULL) {
      if ($role->hasPermission(Constants::SHARE_PERMISSION) === TRUE) {
        $role->grantPermission(Constants::SHARE_PUBLIC_PERMISSION);
        $role->save();
      }
    }
  }

  return t('Update complete. Note that the entity view page and view lists configurations have been necessarily reset to defaults.');
}

/*----------------------------------------------------------------------
 *
 * 0.6.0 to 0.6.1
 *
 *----------------------------------------------------------------------*/

/**
 * Update default enabled commands.
 *
 * This update adds new menu commands.
 */
function foldershare_update_8003() {
  //
  // Enable new menu commands.
  // -------------------------
  // Insure the new menu commands are enabled by default.
  $currentCommands = Settings::getCommandMenuAllowed();
  if (in_array('foldersharecommand_move_as_admin', $currentCommands) === FALSE) {
    $currentCommands[] = 'foldersharecommand_move_as_admin';
  }

  if (in_array('foldersharecommand_move_on_rootlist', $currentCommands) === FALSE) {
    $currentCommands[] = 'foldersharecommand_move_on_rootlist';
  }

  Settings::setCommandMenuAllowed($currentCommands);
}

/**
 * Flush all caches.
 */
function foldershare_update_8004() {
  // This update is intentionally empty. Its effect is to cause all
  // caches to be flushed.
  return t('Update complete.');
}

/*----------------------------------------------------------------------
 *
 * 0.6.1 to 0.6.2
 *
 *----------------------------------------------------------------------*/

/**
 * Update views and add an internal entity type for scheduled tasks.
 *
 * This update fixes a problem with views that allowed some
 * displays to show hidden items. It also installs a new internal entity type
 * that manages scheduled background tasks.
 *
 * Note that the views lists must be reset.  Any site customizations will
 * be lost.
 */
function foldershare_update_8005() {
  //
  // Install FolderShareScheduledTask.
  // ---------------------------------
  // Get the definition for the new FolderShareScheduledTask entity
  // and install it.
  $updateManager = \Drupal::entityDefinitionUpdateManager();
  if ($updateManager->getEntityType('foldershare_scheduledtask') === NULL) {
    $typeManager = \Drupal::entityTypeManager();
    $definition = $typeManager->getDefinition(
      'foldershare_scheduledtask',
      FALSE);

    if ($definition !== NULL) {
      $updateManager->installEntityType($definition);
    }
  }

  //
  // Reset views.
  // ------------
  // All views now check the 'systemhidden' flag.
  ConfigurationUtilities::revertConfiguration(
    'view',
    Constants::VIEW_LISTS);

  return t('Update complete. Note that the view lists configurations have been necessarily reset to defaults.');
}

/*----------------------------------------------------------------------
 *
 * 0.6.2 to 0.6.3
 *
 *----------------------------------------------------------------------*/

/**
 * Update entity definition for internal scheduled tasks.
 *
 * This update reduces the visibility of the internal scheduled
 * task entity. This makes its fields unavailable on pages, through entity
 * references, or within views.
 */
function foldershare_update_8006() {
  // This update is intentionally empty. Its effect is to cause all
  // caches to be flushed, which will pick up changes to the entity type.
  return t('Update complete.');
}

/*----------------------------------------------------------------------
 *
 * 0.6.3 to 0.6.4
 *
 *----------------------------------------------------------------------*/

/**
 * Update entity definition for FolderShare entities.
 *
 * This update disables the static and render cache for
 * FolderShare entities.
 */
function foldershare_update_8007() {
  // This update is intentionally empty. Its effect is to cause all
  // caches to be flushed, which will pick up changes to the entity type.
  return t('Update complete.');
}

/*----------------------------------------------------------------------
 *
 * 0.6.4 to 0.6.5
 *
 *----------------------------------------------------------------------*/

/**
 * Update enabled menu commands.
 *
 * This update disables the compress and uncompress commands,
 * which are in development and not yet ready for production sites.
 */
function foldershare_update_8008() {
  // To disable the compress and uncompress commands, we need to force
  // an enable of command menu restrictions, and then intentionally not
  // list these commands in the list of allowed commands.
  Settings::setCommandMenuRestrict(TRUE);

  $allowedCommands = Settings::getCommandMenuAllowed();
  $keepCommands = [];

  foreach ($allowedCommands as $command) {
    switch ($command) {
      case 'foldersharecommand_archive':
      case 'foldersharecommand_unarchive':
        break;

      default:
        $keepCommands[] = $command;
        break;
    }
  }

  Settings::setCommandMenuAllowed($keepCommands);

  return t('Update complete.');
}

/*----------------------------------------------------------------------
 *
 * 0.6.8 to 1.2.0
 *
 *----------------------------------------------------------------------*/

/**
 * Updates to forms, settings, and routes.
 *
 * This update modifies search indexing, forms, usage reporting,
 * logging, hooks, and background task scheduling.  Some routes and services
 * have changed.  REST web services have been removed from this module and
 * are now available via the separate FolderShare REST module.
 */
function foldershare_update_8009() {
  // Search indexing has changed slightly. Mark everything as in need of
  // reindexing.
  ManageSearch::markForReindex();

  // Delete the old REST configuration now that REST has been moved to
  // a separate module.
  $configStorage = \Drupal::service('config.storage');
  $configStorage->delete('rest.resource.entity.foldershare');

  // This update function's existance will trigger a full cache flush.
  // This will pick up new route and services changes.
  return t('Update complete.');
}

/*----------------------------------------------------------------------
 *
 * 1.2.0 to 1.3.0
 *
 *----------------------------------------------------------------------*/

/**
 * Updates the internal usage table schema to increase integer sizes.
 *
 * This update enables usage statistics to better support sites
 * with a very large number of files and/or large files. The usage
 * statistics table is cleared and can be rebuilt manually from
 * the usage report page. It will be rebuilt automatically if updates are
 * scheduled.
 */
function foldershare_update_8010() {
  $connection = Database::getConnection();
  $name       = ManageUsageStatistics::USAGE_TABLE;
  $newSchema  = ManageUsageStatistics::getSchema()[$name];

  // Temporarily disable automatic usage table updates.
  $updateInterval = Settings::getUsageUpdateInterval();
  Settings::setUsageUpdateInterval('manual');
  ManageUsageStatistics::scheduleUsageUpdate();

  // Drop the old usage statistics table. If it doesn't exist, the function
  // returns immediately, which is OK.
  $connection->schema()->dropTable($name);

  // Create the new usage statistics table. This will throw an exception if:
  // - The table already exists.
  // - The SQL generated by Drupal core is invalid.
  //
  // We already know the table doesn't exist because it was dropped above.
  // And it is very unlikely that the schema-to-SQL generator will produce
  // bogus SQL.
  //
  // But if an exception does occur, let it end this update.
  $connection->schema()->createTable($name, $newSchema);

  // Mark the table as un-updated.
  \Drupal::state()->set('foldershare.usage_last', 'never');

  // Resume automatic usage table updates.
  Settings::setUsageUpdateInterval($updateInterval);
  ManageUsageStatistics::scheduleUsageUpdate();

  // This update function's existance will trigger a full cache flush.
  // This will pick up new route and services changes.
  return t('Update complete.');
}

/**
 * Updates the FolderShare 'name' field to make it case sensitive.
 *
 * This update upgrades file and folder names to be case sensitive
 * and better match the abilities of modern file systems.
 */
function foldershare_update_8011() {
  $database  = \Drupal::database();
  $tableName = FolderShare::BASE_TABLE;
  $entityId  = FolderShare::ENTITY_TYPE_ID;
  $bundleId  = $entityId;

  //
  // Setup.
  // ------
  // Try to insure that we are not interrupted.
  LimitUtilities::setUnlimited();
  @ignore_user_abort(TRUE);

  // Get the current entity definition and the name of the ID field.
  $entityTypeManager = \Drupal::entityTypeManager();
  $definition = $entityTypeManager->getDefinition($entityId);
  $idKey = $definition->getKey('id');

  // Get the original definition for the "name" field.
  $definitionManager = \Drupal::entityDefinitionUpdateManager();
  $originalNameDefinition = $definitionManager->getFieldStorageDefinition(
    'name',
    $entityId);
  $settings = $originalNameDefinition->getSettings();

  //
  // Update the "name" field, if needed.
  // -----------------------------------
  // The old "name" field was mistakenly case insensitive. If automatic
  // entity updates have not corrected this, then we need to do it manually.
  if ($settings['case_sensitive'] === FALSE ||
      $settings['is_ascii'] === TRUE) {
    // Get the original display options.
    $originalDisplayOptions = $originalNameDefinition->getDisplayOptions('view');

    // Create a new case sensitive non-ASCII "name" field definition.
    $newNameDefinition = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of the file or folder.'))
      ->setRequired(TRUE)
      ->setSettings([
        'default_value'   => 'Item',
        'max_length'      => FolderShare::MAX_NAME_LENGTH,
        'text_processing' => FALSE,
        'case_sensitive'  => TRUE,
        'is_ascii'        => FALSE,
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', $originalDisplayOptions)
      ->setDisplayConfigurable('form', FALSE);

    // Query ALL FolderShare entities and save their current names,
    // indexed by their entity IDs. This could be a long list.
    $savedNames = $database->select($tableName)
      ->fields(
        $tableName,
        [
          $idKey,
          'name',
        ])
      ->execute()
      ->fetchAllKeyed();

    // Clear ALL names! This is needed before we can uninstall
    // the field.
    $database->update($tableName)->fields(['name' => NULL])->execute();

    // Uninstall the old "name" field.
    $definitionManager->uninstallFieldStorageDefinition($originalNameDefinition);

    // Install the new "name" field definition.
    $definitionManager->installFieldStorageDefinition(
      'name',
      $entityId,
      $bundleId,
      $newNameDefinition);

    // Restore the saved names.
    foreach ($savedNames as $id => $name) {
      $database->update($tableName)
        ->fields(['name' => $name])
        ->condition($idKey, $id)
        ->execute();
    }
  }

  // This update function's existance will trigger a full cache flush.
  return t('Update complete.');
}

/**
 * Updates the internal task table to add a 'flags' field.
 *
 * This update improves management of background tasks used to complete
 * long operations in the background.
 */
function foldershare_update_8012() {
  //
  // Setup.
  // ------
  // Below we add a new "flags" field to the task queue entity. This does
  // not require that we drain the task queue first. However, it is a good
  // idea to try and insure that we are not interrupted and to disable
  // the task queue.
  LimitUtilities::setUnlimited();
  @ignore_user_abort(TRUE);

  FoldershareScheduledTask::setTaskExecutionEnabled(FALSE);

  $connection = Database::getConnection();
  $schema = $connection->schema();

  //
  // Add "flags" field to tasks, if needed.
  // --------------------------------------
  // Update the base field definition. These definitions are copied from
  // the entity's base field definition for new installs.
  if ($schema->fieldExists(FolderShareScheduledTask::BASE_TABLE, 'flags') === FALSE) {
    $definition = BaseFieldDefinition::create('integer')
      ->setLabel(t('Flags'))
      ->setDescription(t("Flags indicating the task type."))
      ->setRequired(FALSE)
      ->setDefaultValue(0)
      ->setDisplayConfigurable('view', FALSE)
      ->setDisplayConfigurable('form', FALSE);

    // Add the field.
    $updateManager = \Drupal::entityDefinitionUpdateManager();
    $updateManager->installFieldStorageDefinition(
      'flags',
      FolderShareScheduledTask::ENTITY_TYPE_ID,
      FolderShareScheduledTask::ENTITY_TYPE_ID,
      $definition);
  }

  // Set all tasks to a default "flags" value of 0.
  $connection->update(FolderShareScheduledTask::BASE_TABLE)
    ->fields([
      'flags' => (int) 0,
    ])
    ->execute();

  // This update function's existance will trigger a full cache flush.
  return t('Update complete.');
}

/**
 * Updates the internal task table to use a larger 'parameters' field.
 *
 * This update improves handling for large background tasks that operate
 * on deep or wide folder trees.
 */
function foldershare_update_8013() {
  //
  // Setup.
  // ------
  // Try to insure that we are not interrupted, disable all known repeating
  // tasks, and drain the task queue.
  LimitUtilities::setUnlimited();
  @ignore_user_abort(TRUE);

  // Temporarily disable the search indexing task, if any.
  $originalSearchIndexInterval = Settings::getSearchIndexInterval();
  if ($originalSearchIndexInterval !== 'system') {
    Settings::setSearchIndexInterval('system');
    ManageSearch::scheduleSearchIndexing();
  }

  // Temporarily disable the usage updating task, if any.
  $originalUsageUpdateInterval = Settings::getUsageUpdateInterval();
  if ($originalUsageUpdateInterval !== 'manual') {
    Settings::setUsageUpdateInterval('manual');
    ManageUsageStatistics::scheduleUsageUpdate();
  }

  // Try to drain the task queue. This could fail if there are repeating
  // tasks. However, the task queue is supposed to be exclusive to
  // this module, there are only two known repeating tasks, and above we
  // disabled both of them. So there should be no errors. And if there are,
  // then we cannot be responsible for something else sneaking into the
  // module's private task queue.
  FolderShareScheduledTask::setTaskExecutionEnabled(TRUE);
  FolderShareScheduledTask::finishTasks();

  // Disable further task execution.
  FoldershareScheduledTask::setTaskExecutionEnabled(FALSE);

  //
  // Update the "parameters" field, if needed.
  // -----------------------------------------
  // The old "parameters" field was a case insensitive string limited to
  // 255 characters. It is used to hold JSON-encoded task parameters. On
  // large operations, this could overflow so we need to bump up the size
  // and certainly mark it as case sensitive so that JSON always works
  // properly.
  $database  = \Drupal::database();
  $tableName = FolderShareScheduledTask::BASE_TABLE;
  $entityId  = FolderShareScheduledTask::ENTITY_TYPE_ID;
  $bundleId  = $entityId;

  // Get the current entity definition and the name of the ID field.
  $entityTypeManager = \Drupal::entityTypeManager();
  $definition = $entityTypeManager->getDefinition($entityId);
  $idKey = $definition->getKey('id');

  // Get the original definition for the "parameters" field.
  $definitionManager = \Drupal::entityDefinitionUpdateManager();
  $originalParametersDefinition = $definitionManager->getFieldStorageDefinition(
    'parameters',
    $entityId);
  $settings = $originalParametersDefinition->getSettings();
  $fieldType = $originalParametersDefinition->getType();

  if ($fieldType === 'string' ||
      $settings['case_sensitive'] === FALSE) {
    // Create a new long string and case insensitive "parameters"
    // field definition.
    $newParametersDefinition = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Task parameters'))
      ->setDescription(t('The JSON parameters for the task.'))
      ->setRequired(FALSE)
      ->setSettings([
        'case_sensitive' => TRUE,
      ])
      ->setDisplayConfigurable('view', FALSE)
      ->setDisplayConfigurable('form', FALSE);

    // Query ALL tasks and save their current parameters, indexed by their
    // entity IDs. Since we have drained the task queue above and stopped
    // all repeating tasks, this should be empty. But be paranoid.
    $savedParameters = $database->select($tableName)
      ->fields(
        $tableName,
        [
          $idKey,
          'parameters',
        ])
      ->execute()
      ->fetchAllKeyed();

    // Clear ALL parameters! This is needed before we can uninstall
    // the field. Again, the task queue should be empty so this should
    // do nothing.
    $database->update($tableName)->fields(['parameters' => NULL])->execute();

    // Uninstall the old "parameters" field.
    $definitionManager->uninstallFieldStorageDefinition(
      $originalParametersDefinition);

    // Install the new "parameters" field definition.
    $definitionManager->installFieldStorageDefinition(
      'parameters',
      $entityId,
      $bundleId,
      $newParametersDefinition);

    // Restore the saved parameters. Again, there should have been no tasks
    // in the queue, so there should be no saved parameters to restore.
    foreach ($savedParameters as $id => $parameters) {
      $database->update($tableName)
        ->fields(['parameters' => $parameters])
        ->condition($idKey, $id)
        ->execute();
    }
  }

  //
  // Finish.
  // -------
  // Re-enable the task queue and restore the run mode for all known
  // repeating tasks.
  FoldershareScheduledTask::setTaskExecutionEnabled(TRUE);

  // Restore the usage reporting task run mode.
  if ($originalUsageUpdateInterval !== 'manual') {
    Settings::setUsageUpdateInterval($originalUsageUpdateInterval);
    ManageUsageStatistics::scheduleUsageUpdate();
  }

  // Restore the search indexing task run mode.
  if ($originalSearchIndexInterval !== 'system') {
    Settings::setSearchIndexInterval($originalSearchIndexInterval);
    ManageSearch::scheduleSearchIndexing();
  }

  // This update function's existance will trigger a full cache flush.
  return t('Update complete.');
}
